////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef BLINN_MICRO_H
#define BLINN_MICRO_H


#include "bxdfbase.h"
#include "bxdfmicrofacet.h"
#include "microfacetdistribution.h"

class CBlinnMicroDist : public CMicrofacetDistribution
{
public:
  CBlinnMicroDist( double Exp);
  virtual ~CBlinnMicroDist(void);

  void  SetExp( double Exp ) { m_Exponent = Exp; };
  double GetExp() const { return m_Exponent; };

  // Blinn public Methods
  double D(const Vector& HalfVec, const Vector& Normal) const {
    double CosThetaHalf = fabs( Normal.dot( HalfVec ) );
    return ( m_Exponent + 2.0 ) / PI2 * pow( std::max( 0.0, CosThetaHalf ), m_Exponent );
  }

	virtual void  Sample_f( const Vector& Out, Vector& In, const Vector& Normal, double u1, double u2, float& Pdf ) const;
	virtual float GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const;

private:
  double m_Exponent;
};

#endif
// vim:ts=2
