////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef LA_FORTUNE_H
#define LA_FORTUNE_H

#include "bxdfbase.h"

class CBxDFLaFortune :  public CBxDFBase
{
public:
  CBxDFLaFortune( const Color& Diffuse, unsigned int NrOfLobes, const Color* pX, const Color* pY, const Color* pZ, const Color* pExp, CBxDFBase::BxDFType Type);
  virtual ~CBxDFLaFortune(void);

  // f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
  virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );

  // Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
  // BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
  // continue recursively.
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );
  virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const;

protected:
  unsigned int m_NrOfLobes;

  const Color& m_Diffuse;
  const Color* m_pX;
  const Color* m_pY;
  const Color* m_pZ;
  const Color* m_pExp;
};


#endif
// vim:ts=2
