////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef BXDF_SPEC_REFL_H
#define BXDF_SPEC_REFL_H

#include "bxdfbase.h"
#include "bxdffresnel.h"

class CFresnelDiel;
class CFresnel;

class CBxDFSpecReflection : public CBxDFBase
{
public:
  CBxDFSpecReflection(CFresnel* pFresnel );
  virtual ~CBxDFSpecReflection(void);

  // f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
  virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );

  // Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
  // BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
  // continue recursively.
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );
	virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const {
		UNUSED( In ); UNUSED( Out ); UNUSED( Normal );
		return 1.0;
	};

protected:
  virtual double Sample_f_Reflection( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler );

protected:
  CFresnel*  m_pFresnel;
};



#endif
// vim:ts=2
