////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#ifndef SPEC_TRANSMISSION_H
#define SPEC_TRANSMISSION_H

#include "bxdfbase.h"
#include "bxdffresnel.h"

class CFresnelDiel;

class CBxDFSpecTransmission : public CBxDFBase
{
public:
  explicit CBxDFSpecTransmission( const Color& TransmissionColor = Color( 1.0, 1.0, 1.0 ) );
  virtual ~CBxDFSpecTransmission(void);

  // f( Out, In) evaluates the BxDF for the in & outgoing pair of rays
  virtual double f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 ) {
    UNUSED( Out ); UNUSED( In ); UNUSED( Normal ); UNUSED( RenderMat ); UNUSED( LightColor ); UNUSED( Power ); UNUSED( Shade );
    return 0.0;
  }

  // Sample_f samples an incoming ray matching to the outgoing ray + its Pdf. Furthermore it evaluates the
  // BxDF according to In & Out and returns the color. This is used for reflection, refraction etc. which will
  // continue recursively.
	virtual double Sample_f( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler, BxDFType Type, const Color& LightColor = Color( 1.0, 1.0, 1.0 ), double Power = 1.0, float Shade = 1.0 );
	virtual double GetPDF( const Vector& Out, const Vector& In, const Vector& Normal ) const {
		UNUSED( In ); UNUSED( Out ); UNUSED( Normal );
		return 1.0;
	};

protected:
  virtual double Sample_f_Transmission( const Ray& Out, Ray& In, float& Pdf, const Vector& Normal, const Material& RenderMat, Random& Sampler );

protected:
  CFresnelDiel*   m_pFresnel;
  Color           m_TransmissionColor;

};

#endif
// vim:ts=2
