////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include "bxdftoon.h"

CBxDFToon::CBxDFToon(void) : CBxDFBase( static_cast<BxDFType>( BSDF_DIFFUSE | BSDF_SPECULAR ) )
{
}

CBxDFToon::~CBxDFToon(void)
{
}

////////////////////////////////////////////////////////////////////////////////////////////////////////////

double CBxDFToon::f( const Ray& Out, const Ray& In, const Vector& Normal, const Material& RenderMat, const Color& LightColor, double Power, float Shade /*= 1.0*/ )

////////////////////////////////////////////////////////////////////////////////////////////////////////////
{
  UNUSED( LightColor ); UNUSED( Power ); UNUSED( Shade );

  double          Rho = 0.0;
  Vector          L = In.dir();        // light vector
  Vector          OutDir = -Out.dir();
  const float     cDSize   = 1.5f;          // some shader parameters which we fix for the moment
  const float     cDSmooth = 1.5f;
  const float     cEdge   = 0.3f;

  // calculate diffuse shading
  double Dot = Normal.dot( L );

  if(Dot > SMALL3D ) {
    double DiffuseCoeff = ( 1.0 - RenderMat.specular() );
    double CosTIn = Dot;

    if( CosTIn <= 0.0 )
      return 0;

    double Diff, Angle = acos( CosTIn );
    if( Angle < cDSize )
      Diff = 1.0;
    else if( ( Angle >= (cDSize + cDSmooth)) || ( cDSmooth == 0.0) )
      Diff = 0.0;
    else
      Diff = 1.0 - ((Angle - cDSize) / cDSmooth );

    // extra edge factor
    double e = OutDir.dot( Normal );
    if( e < cEdge)
      e = 0.0;
    else
      e = 1.0;

    // set diffuse component
    Rho = DiffuseCoeff / PI * (Diff * e ) / CosTIn;

    //
    // determine specular component
    //
    if( RenderMat.specular() > 0) {
      Vector H( L + OutDir );
      H.normalize();

      double hn = Normal.dot( H );
      if( hn <= 0.0)
        return Rho;

      double Angle2 = acos(hn);
      if( Angle2 < cDSize)
        hn = 1.0;
      else if( ( Angle2 >= (cDSize + cDSmooth)) || (cDSmooth == 0.0))
        hn = 0.0;
      else
        hn = 1.0 - ((Angle2 - cDSize)/ cDSmooth);

      Rho = Rho + RenderMat.specular() / PI * hn / CosTIn;
    }
  }


  return Rho;
}
// vim:ts=2
