////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#ifndef FARM_H
#define FARM_H

#include <string>

#include "farmsurfaceintegrator.h"
#include "voxelintegrator.h"
#include "graph/color.h"
#include "configparser.h"

// class D3Layer;
// class VVoxel;

class CFARM
{
public:
	CFARM();
	~CFARM();

	// basic initialization
	void Init( D3Layer* pVisLayer, VVoxel* pVoxelLayer, Geometry* pGeometry);

	// read the optional configuration file
	bool		ReadConfigFile( const std::string& Path );

	// check if config file has been read already
	bool ConfigRead() const { return m_bConfigRead; };

	// access surface integrator
	CSurfaceIntegrator* GetSurfaceInt() { return &m_SurfaceIntegrator; }

	// get/set render mode quality (speed vs. quality). default = speed
	int			GetRenderMode() const { return m_SurfaceIntegrator.GetRenderMode(); };
	void		SetRenderMode( int RenderMode, int QualityLevel = 0 ) { m_SurfaceIntegrator.SetRenderMode( RenderMode, QualityLevel );
																																	m_VoxelIntegrator.SetRenderMode( RenderMode, QualityLevel ); };

	// treat passed shading ray
	Color		Shade( GeometryEngine* pGeoEngine, Ray* pShadingRay, const int RenderPass, int Light0 );

	// treat passed shading ray
	Color		ShadeVoxel( GeometryEngine* pGeoEngine, Ray* pShadingRay, const int RenderPass, int Light0 );

	// check if the voxel hit by the passed ray is rendered (semi) transparent
	bool		IsVoxelDrawnTransparent( Ray* pShadingRay );

	// return the transparency [0,1] of a voxel associated with the shading ray
	Color		GetVoxelTransparency( Ray* pShadingRay );

	// return shading factor for data [0 = black, 1 = white]
	double	ShadeData( GeometryEngine* pEngine, Ray* pRay, const int RenderPass, int Light0 );

	// return the configuration objection
	CConfigParser& GetConfig()  { return m_ConfigParser; };


protected:
	CSurfaceIntegrator	m_SurfaceIntegrator;
	CVoxelIntegrator		m_VoxelIntegrator;

	CConfigParser				m_ConfigParser;
	bool								m_bConfigRead;

	// configuration flags
	bool								m_bShadeData;

	D3Layer*						m_pVisLayer;
	VVoxel*							m_pVoxelLayer;
};

#endif
// vim:ts=2
