////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// F A R M     -  Flair advanced rendering module
//
// Copyright European Organization for Nuclear Research( CERN )
// All rights reserved
//
// Authors: Christian.Theis@cern.ch
//			    Vasilis.Vlachoudis@cern.ch
//
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#pragma once
#ifndef SURFACE_INTEGRATOR_H
#define SURFACE_INTEGRATOR_H

#include "baseintegrator.h"
#include "graph/color.h"
#include "matphong.h"
#include "lightmap.h"
#include "biasedlightmap.h"

#include <vector>

class D3Layer;

class CSurfaceIntegrator : public CBaseIntegrator
{
public:
	CSurfaceIntegrator();
	virtual ~CSurfaceIntegrator( void );

	// initialize options configured in the optional config object in FARM
	void InitFromConfig();

	// main integrator routine returning the luminance for the passed ray & rendering pass
	virtual Color Li( GeometryEngine* pEngine, Ray* pRay, const int RenderPass, int Light0 );

	// configure ambient occlusion
	void SetAOCParameters( double MinIntensity, double MaxIntensity, double MaxDist, int NrOfSamples )
	{
		m_AOCMinIntensity = MinIntensity;
		m_AOCMaxIntensity = MaxIntensity;
		m_AOCMaxDist      = MaxDist;
		m_NrOfAOCSamples  = NrOfSamples;
	}

	void SetAOCSamples(int NrOfSamples) { m_NrOfAOCSamples = NrOfSamples; }
	int  GetAOCSamples() const          { return m_NrOfAOCSamples; }

	// retrieve the light intensity at the hit point -> used to apply shadows for data that is displayed
	double	GetLightIntensityAtHit( GeometryEngine* pEngine, Ray* pRay, const int RenderPass, int Light0 ) { return ShadeHitPoint( pEngine, pRay, RenderPass, Light0 ); };

protected:
	// setup bi-directional scattering & reflection LUT
	void    SetupBxDFLUT();

	// ambient occlusion integrator
	double CalcAmbientOcclusion( GeometryEngine* eng, Ray* pixelRay, int nrOfSamples, const double cMaxDist, const double cMinIntensity, const double cMaxIntensity, bool bScramble  = false  );

	// incoming reflection
	virtual Color Li_Reflection( GeometryEngine* pEngine, Ray* pRay, const int RenderPass, int Light0 );

	// refraction & transmission
	virtual Color Li_Transmission( GeometryEngine* pEngine, Ray* pRay, const int RenderPass, int Light0 );

	// environmental light maps
	virtual Color Li_LightMap( GeometryEngine* pEngine, Ray* pRay );

	// calculate shading intensity of the current hit point
	double				ShadeHitPoint( GeometryEngine* pEngine, Ray* pRay, const int RenderPass, int Light0 );

protected:
	std::vector<CPhysMatInterface*> m_BxDFLUT;
	CPhysMatInterface* m_pBxDF;

	double		m_AOCMinIntensity;
	double		m_AOCMaxIntensity;
	double		m_AOCMaxDist;
	int				m_NrOfAOCSamples;
	int				m_MaxAOCDepth;

	bool			m_bUseLightMap;

	CBiasedLightMap*	m_pLightMap;
	int								m_NrOfLightMapSamples;
};

#endif
// vim:ts=2
