/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:        26 Feb 2013
 */

#include "material.h"

#include <map>
#include <iostream>
#include <exception>

// Material bxdf dictionary
// BSDF (bi-direction scattering distribution function)
// BRDF (bi-directional reflection distribution function)
// BDTF (bi-directional transmission distribution function)
// BSSDF (bi-direction sub-surface scattering distribution function)
std::map<const std::string, int> Material::bxdf_map = {
		{"Default"              ,  0},
		{"Plastic smooth"       ,  1},
		{"Plastic rough"        ,  2},
		{"Plastic rough 2"      ,  3},
		{"Shiny metal"          ,  4},
		{"Shiny metal rough"    ,  5},
		{"Shiny metal rough 2"  ,  6},
		{"Brushed metal"        ,  7},
		{"Substrate rough"      ,  8},
		{"Substrate polished"   ,  9},
		{"Metallic paint"       , 10},
		{"Skin"                 , 11},
		{"Minnaert"             , 12},
		{"Toon"                 , 13},
		{"Translucent"          , 14},
		{"Refractive glass"     , 15},
		{"Clear glass/Water"    , 16},
		{"Dispersive glass"     , 17},
		{"High dispersive glass", 18},
};

/* -------------- Material ---------------- */
Material::Material(const std::string& n)
	: _name(n),
	  _density(0.0),
	 _Z(0),
	 _A(0),
	 _weight(0.0),

	 _diff(0x000000),
//	 _amb(0x000000),
//	 _cshine(0x000000),
	 _spec(0.0),
	 _shine(0.0),
	 _ior(1.0),
	 _fuzz(0.0),
	 _bxdfIndex(0),
	 _flag(MAT_CACHE),
	 _used(false)
{
//	name(n);
//	_density = 0.0;
//	_Z       = 0;
//	_A       = 0;
//	_weight  = 0.0;
//
//	_diff   = 0x000000;
////	_amb    = 0x000000;
////	_cshine = 0x000000;
//	_spec   = 0.0;
//	_shine  = 0.0;
//	_ior    = 1.0;
//	_fuzz   = 0.0;
//	_flag   = MAT_CACHE;
//	_used   = false;
} // Material

/** set bxdf from type name
 * @return true on success
 */
bool Material::bxdf(const char* type)
{
	if (type[0]=='\0') {
		_bxdfIndex = 0;
		return true;
	}
	auto search = bxdf_map.find(type);
	if (search == bxdf_map.end())
		return false;
	_bxdfIndex = search->second;
	return true;
} // bxdf

/** @return the bxdf type name as string */
std::string Material::bxdfType() const
{
	for (auto x : bxdf_map)
		if (x.second == _bxdfIndex)
			return x.first;
	return std::string("unknown");
} // bxdfType

/* -------------- operator << ---------------- */
std::ostream& operator << (std::ostream& os, const Material& mat)
{
	os << "material \"" << mat.name() << "\" {" << std::endl;
	os << "\tdiffuse=" << mat.diffuse() << std::endl;
	os << "\tspecular=" << mat.specular() << std::endl;
//	os << "\tambient=" << mat.ambient() << std::endl;
//	os << "\ttransparent=" << mat.transparent() << std::endl;
//	if (RED(mat.shineColor())>0)
//		os << "\tshine=" << mat.shineColor() << std::endl;
//	else
//		os << "\tshine=" << mat.shineExp() << std::endl;
	os << "\tior=" << mat.ior() << std::endl;
	os << "\tfuzz=" << mat.fuzz() << std::endl;
	if (mat.flags() & MAT_CACHE)
		os << "\tCACHE" << std::endl;
	if (mat.flags() & MAT_NO_ANTIALIAS)
		os << "\tNO_ANTIALIAS" << std::endl;
	os << '}';
	return os;
} /* operator << */
