/*
 * Copyright European Organization for Nuclear Research (CERN)
 * All rights reserved
 *
 * Please look at the supplied documentation for the user's
 * license
 *
 * Author:	Vasilis.Vlachoudis@cern.ch
 * Date:        19 Feb 2013
 */

#ifndef __NOX_H
#define __NOX_H

#include <vector>
#include <fstream>

#include "viewer/viewer.h"
#include "geometry/geometry.h"

class Matrix4;

struct region_material_t {
	int	region_id;
	dword	color;
	int	alpha;
};

/** Nox Geometry/Viewer parser
 */
class NoxGeometry {
public:
	Geometry	geometry;
	GeometryKernel	kernel;
	GeometryViewer	viewer;
	std::vector<region_material_t> region_materials;
	bool		dump;

public:
	NoxGeometry(const char *filename=NULL);
	bool	load(const char *filename);
	void	cleanup();

private:
	bool	parse(std::ifstream& input);
	GBody*	parseBody();
	GRegion* parseRegion();
	bool	parseRegionExpr(GRegion* region, std::string* line=nullptr);
	void	parseVoxel();
	void	assignRegionMaterials();
	static void parseMatrix(Matrix4& matrix);
}; // class NoxGeometry
#endif
